/**************************************************************/
/* This program calculates the rescaled adjusted range        */
/* statistic (R/S statistic) for a data set. That is, it      */
/* computes R(t(i), n)/S(t(i), n) values for different n's.   */
/*                                                            */
/* The input of this program will be packet counts (a given   */
/* sample of N observations). We subdivide the whole sample   */
/* into K non-overlapping blocks of N/K observations each,    */
/* and compute the rescaled adjusted range statistic          */
/* r(t(i), n)/s(t(i), n) for each of the new block            */
/* "starting points" t(i) with (t(i) - 1) + n <= N, for       */
/* i = 1, 2, ... and (i - 1) * (N / K) + 1 < n < N.           */
/*                                                            */
/* For the given value of N observations, the block size K    */
/* can be specified by the user. Default value for K is 1.    */
/*                                                            */
/* Usage: gcc -o rs rs.c -lm                                  */
/*        rs [K] < infile > outfile                           */
/*                                                            */
/**************************************************************/ 
  
/* Usual include files */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Maximum length of time series we can analyze */
#define MAX_OBSERVATIONS 171001

/* Debugging flags */
/* #define DEBUG1 1  */
/* #define DEBUG2 1 */
/* #define DEBUG3 1 */
#define OLD_WAY 1
/* #define NEW_WAY 1 */
/* #define NEWEST_WAY 1 */
#define FAST 1

/* Data structures to hold the original time series */
double X[MAX_OBSERVATIONS];

/* Data structures to hold calculated statistics */
double sumofX[MAX_OBSERVATIONS];
double W[MAX_OBSERVATIONS];
double W2[MAX_OBSERVATIONS];
double R[MAX_OBSERVATIONS];
double S[MAX_OBSERVATIONS];

main(int argc, char *argv[])
  {
    double sum, mean, var, sdev;
    int N, K;
    int i, j, k, n, t; 
    double max, min;
    int imax, imin;
    float current;
    int blocksize;

    /* Set default number of intervals */
    K = 1;

    /* Check for command line arguments */
    if (argc > 1)
      sscanf(argv[1], "%d", &K);

    if( K <= 0 )
      {
	printf("Illegal number of intervals K: %d\n", K);
	printf("Usage: %s [K] < infile > outfile\n", argv[0]);
	exit(1);
      }

    /* Read in and store the values in the time series. */
    /* Note: the X's are stored starting at index 1,    */
    /* not index 0. This wastes one storage space, but  */
    /* makes it easier to understand the math later.    */
    N = 1;
    while( scanf("%f", &current) != EOF )
      {
	if( N >= MAX_OBSERVATIONS )
	  {
	    printf("Whoa! Too many observations!!\n");
	    exit(0);
	  }
	X[N] = current;
	N++;
      }

   /* Compute the mean of the observations */
    sum = 0.0;
    for( i = 1; i <= N; i++ )
      {
	sum += X[i];
      }
    mean = sum / N;

   /* Compute S, the standard deviation of the observations */
    sum = 0.0;
    for( i = 1; i <= N; i++ )
      {
	sum += (X[i] - mean) * (X[i] - mean);
      }
    var = sum / N;
    sdev = sqrt(var);

    /* Compute the Wi's */
    sum = 0.0;
    for( i = 1; i <= N; i++ )
      {
	sum += X[i];
	W[i] = sum - i * mean;
      }

    /* Determine size of each block */
    blocksize = N / K;

    /* Compute R/S statistic for each legal value of n  */
    /* from each starting point.                        */
    for( k = 1; k <= K; k++ ) /* number of blocks */
      {

	/* Determine starting point for block k */
	t = (k - 1) * blocksize + 1;

	/* Leland suggests starting with n around 10 */
	n = 1;
	n = 5;
	n = 10;
	while( n < N - t + 1 )
	  {
	    /* Compute the mean of the first n observations */
	    /* starting from the starting point t.          */
	    sum = 0.0;
	    for( i = 1; i <= n; i++ )
	      {
		sum += X[t+i-1];
      }
	    mean = sum / n;

	    /* Compute S, the standard deviation of these observations */
	    sum = 0.0;
	    for( i = 1; i <= n; i++ )
	      {
		sum += (X[t+i-1] - mean) * (X[t+i-1] - mean);
	      }
	    var = sum / n;
	    sdev = sqrt(var);

	    /* save it for later use */
	    S[n] = sdev;
	    
	    /* Compute the Wi's */
	    sum = 0.0;
	    for( i = 1; i <= n; i++ )
	      {
		sum += X[t+i-1];
		/* use the first i values starting with X[t] */
		/* store result in W[i] */
		W[i] = sum - i * mean;
	      }
	    /* find largest Wi value */
	    max = 0;
	    imax = -1;
	    for( i = 1; i <= n; i++ )
	      {
		if( W[i] > max )
		  {
		    max = W[i];
		    imax = i;
		  }
	    /* find smallest Wi value */
	    min = 0;
	    imin = -1;
	    for( i = 1; i <= n; i++ )
	      {
		if( W[i] < min )
		  {
		    min = W[i];
		    imin = i;
		  }
	      }
	
	    /* Compute R and store it */
	    R[n] = max - min;


	    /* print out R/S statistic */
	    if( S[n] > 0.0 )
	      printf("%d %f\n", n, R[n]/S[n]);

	    /* get next value of n */
	    /* go by bigger steps to get visual approximation */
	    /* this speeds up execution by a factor of 100 or more */
	    if( n < 10 )
	      n += 1;
	    else if( n < 20 )
	      n += 2;
	    else if( n < 50 )
	      n += 5;
	    else if( n < 100 )
	      n += 10;
	    else if( n < 200 )
	      n += 20;
	    else if( n < 500 )
	      n += 50;
	    else if( n < 1000 )
	      n += 100;
	    else if( n < 2000 )
	      n += 200;
	    else if( n < 5000 )
	      n += 500;
	    else if( n < 10000 )
	      n += 1000;
	    else if( n < 20000 )
	      n += 2000;
	    else if( n < 50000 )
	      n += 5000;
	    else if( n < 100000 )
	      n += 10000;
	    else if( n < 200000 )
	      n += 20000;
	    else if( n < 500000 )
	      n += 50000;
	    else n += 100000;
	  } /* end while n */
      } /* end for k */
  }

